@@ -6,7 +6,7 @@ from hwilib.devices.trezor import TrezorClient
 class Trezor(HWIDevice):
     device_type = DeviceTypes.TREZOR
     name = "Trezor"
-    icon = "trezor_icon.svg"
+    icon = "img/devices/trezor_icon.svg"
 
     supports_hwi_toggle_passphrase = True
     supports_hwi_multisig_display_address = True
