@@ -1,14 +1,6 @@
 import json, os, random, requests
-from flask import (
-    Blueprint,
-    Flask,
-    jsonify,
-    url_for,
-    redirect,
-    render_template,
-    request,
-    flash,
-)
+from flask import Blueprint, Flask, jsonify, url_for, redirect, render_template, request
+from .server_endpoints import flash
 from flask import current_app as app
 from flask_cors import CORS
 from .hwi_rpc import HWIBridge
@@ -18,7 +10,6 @@ from .helpers import deep_update, hwi_get_config, save_hwi_bridge_config
 hwi_server = Blueprint("hwi_server", __name__)
 CORS(hwi_server)
 rand = random.randint(0, 1e32)  # to force style refresh
-hwi = HWIBridge()
 
 
 @hwi_server.route("/", methods=["GET"])
@@ -63,12 +54,12 @@ def api():
                 )
     try:
         data = json.loads(request.data)
-    except:
+    except Exception as e:
         return (
             jsonify(
                 {
                     "jsonrpc": "2.0",
-                    "error": {"code": -32700, "message": "Parse error"},
+                    "error": {"code": -32700, "message": f"Parse error:{str(e)}"},
                     "id": None,
                 }
             ),
@@ -93,19 +84,17 @@ def api():
                 500,
             )
         data["forwarded_request"] = True
-        requests_session = requests.Session()
+        requests_session = app.specter.requests_session(
+            force_tor=".onion/" in app.specter.hwi_bridge_url
+        )
         requests_session.headers.update({"origin": request.environ["HTTP_ORIGIN"]})
-        if ".onion/" in app.specter.hwi_bridge_url:
-            requests_session.proxies = {}
-            requests_session.proxies["http"] = "socks5h://localhost:9050"
-            requests_session.proxies["https"] = "socks5h://localhost:9050"
         forwarded_request = requests_session.post(
             app.specter.hwi_bridge_url, data=json.dumps(data)
         )
         response = json.loads(forwarded_request.content)
         return jsonify(response)
 
-    return jsonify(hwi.jsonrpc(data))
+    return jsonify(app.specter.hwi.jsonrpc(data))
 
 
 @hwi_server.route("/settings/", methods=["GET", "POST"])
