@@ -1,14 +1,17 @@
+from gc import callbacks
 import os
 import logging
 import secrets
 import shutil
 
 from ..rpc import get_default_datadir, RPC_PORTS
-from ..specter_error import SpecterError
-from ..persistence import write_node, delete_file
-from ..helpers import alias, load_jsons
+from ..specter_error import SpecterError, SpecterInternalException
+from ..persistence import PersistentObject, write_node, delete_file
+from ..helpers import alias, calc_fullpath, load_jsons
 from ..node import Node
 from ..internal_node import InternalNode
+from ..services import callbacks
+from ..managers.service_manager import ServiceManager
 from ..util.bitcoind_setup_tasks import setup_bitcoind_thread
 
 logger = logging.getLogger(__name__)
@@ -26,21 +29,25 @@ class NodeManager:
         bitcoind_path="",
         internal_bitcoind_version="",
         data_folder="",
+        service_manager=None,
     ):
+        self.nodes = {}
+        # Dict is sth. like: {'nigiri_regtest': <Node name=Nigiri regtest fullpath=...>, 'default': <Node name=Bitcoin Core fullpath=...>}
         self.data_folder = data_folder
         self._active_node = active_node
         self.proxy_url = proxy_url
         self.only_tor = only_tor
         self.bitcoind_path = bitcoind_path
         self.internal_bitcoind_version = internal_bitcoind_version
-        self.update(data_folder)
+        self.service_manager: ServiceManager = service_manager
+        self.load_from_disk(data_folder)
         internal_nodes = [
             node for node in self.nodes.values() if not node.external_node
         ]
         for node in internal_nodes:
             node.start()
 
-    def update(self, data_folder=None):
+    def load_from_disk(self, data_folder=None):
         if data_folder is not None:
             self.data_folder = data_folder
             if data_folder.startswith("~"):
@@ -48,22 +55,43 @@ class NodeManager:
             # creating folders if they don't exist
             if not os.path.isdir(data_folder):
                 os.mkdir(data_folder)
-        nodes = {}
-        nodes_files = load_jsons(self.data_folder, key="name")
+        nodes_files = load_jsons(self.data_folder, key="alias")
         for node_alias in nodes_files:
-            fullpath = os.path.join(self.data_folder, "%s.json" % node_alias)
-            node_class = (
-                Node if nodes_files[node_alias]["external_node"] else InternalNode
-            )
-            nodes[nodes_files[node_alias]["name"]] = node_class.from_json(
-                nodes_files[node_alias],
-                self,
-                default_alias=node_alias,
-                default_fullpath=fullpath,
-            )
-        if not nodes:
+            try:
+                valid_node = True
+                node = PersistentObject.from_json(
+                    nodes_files[node_alias],
+                    self,
+                    default_alias=nodes_files[node_alias]["alias"],
+                    default_fullpath=calc_fullpath(self.data_folder, node_alias),
+                )
+                if (
+                    node.__class__.__module__.split(".")[1] == "specterext"
+                ):  # e.g. cryptoadvance.specterext.spectrum
+                    if self.service_manager:
+                        if not self.service_manager.is_class_from_loaded_extension(
+                            node.__class__
+                        ):
+                            logger.warning(
+                                f"Cannot Load {node} due to corresponding plugin not loaded"
+                            )
+                            continue
+                    else:
+                        logger.warning(
+                            f"Cannot validate Node {node} to be a valid node, skipping"
+                        )
+                        continue
+                self.nodes[node_alias] = node
+                logger.info(f"Loaded Node {self.nodes[node_alias]}")
+            except SpecterInternalException as e:
+                logger.error(f"Skipping node {node_alias} due to {e}")
+
+        if not self.nodes:
             if os.environ.get("ELM_RPC_USER"):
-                self.add_node(
+                logger.debug(
+                    "Creating an external Elements node with the initial configuration."
+                )
+                self.add_external_node(
                     node_type="ELM",
                     name="Blockstream Liquid",
                     autodetect=True,
@@ -73,10 +101,34 @@ class NodeManager:
                     port=7041,
                     host="localhost",
                     protocol="http",
-                    external_node=True,
                     default_alias=self.DEFAULT_ALIAS,
                 )
-            self.add_node(
+            logger.debug(
+                "Creating an external BTC node with the initial configuration."
+            )
+            self.add_external_node(
+                node_type="BTC",
+                name="Bitcoin Core",
+                autodetect=True,
+                datadir=get_default_datadir(),
+                user="",
+                password="",
+                port=8332,
+                host="localhost",
+                protocol="http",
+                default_alias=self.DEFAULT_ALIAS,
+            )
+
+        # Make sure we always have the default node
+        # (needed for the rpc-as-pin-authentication used on Raspiblitz)
+        has_default_node = False
+        for node in self.nodes.values():
+            if node.alias == self.DEFAULT_ALIAS:
+                has_default_node = True
+        # Recreate the default node if it doesn't exist anymore
+        if not has_default_node:
+            logger.debug("Recreating the default node.")
+            self.add_external_node(
                 node_type="BTC",
                 name="Bitcoin Core",
                 autodetect=True,
@@ -86,32 +138,41 @@ class NodeManager:
                 port=8332,
                 host="localhost",
                 protocol="http",
-                external_node=True,
                 default_alias=self.DEFAULT_ALIAS,
             )
-        else:
-            self.nodes = nodes
 
     @property
-    def active_node(self):
+    def active_node(self) -> Node:
         return self.get_by_alias(self._active_node)
 
     @property
-    def nodes_names(self):
-        return sorted(self.nodes.keys())
+    def nodes_names(self) -> list:
+        """Returns a list of the names (not the aliases) of the nodes"""
+        return [node.name for node in self.nodes.values()]
 
-    def switch_node(self, node_alias):
-        # this will throw an error if the node doesn't exist
+    def nodes_by_chain(self, chain: str) -> list:
+        """Returns a list of nodes for a given blockchain"""
+        return [node for node in self.nodes.values() if node.chain == chain]
+
+    def switch_node(self, node_alias: str):
+        # This will throw an error if the node doesn't exist
+        logger.debug(f"Switching from {self._active_node} to {node_alias}.")
         self._active_node = self.get_by_alias(node_alias).alias
 
-    def default_node(self):
+    def default_node(self) -> Node:
         return self.get_by_alias(self.DEFAULT_ALIAS)
 
-    def get_by_alias(self, alias):
-        for node_name in self.nodes:
-            if self.nodes[node_name] and self.nodes[node_name].alias == alias:
-                return self.nodes[node_name]
-        raise SpecterError("Node %s does not exist!" % alias)
+    def get_by_alias(self, alias: str) -> Node:
+        for node in self.nodes.values():
+            if node.alias == alias:
+                return node
+        raise SpecterError("Node alias %s does not exist!" % alias)
+
+    def get_by_name(self, name: str) -> Node:
+        for node in self.nodes.values():
+            if node.name == name:
+                return node
+        raise SpecterError("Node name %s does not exist!" % name)
 
     def update_bitcoind_version(self, specter, version):
         stopped_nodes = []
@@ -131,7 +192,7 @@ class NodeManager:
         for node_alias in stopped_nodes:
             self.get_by_alias(node_alias).start(timeout=60)
 
-    def add_node(
+    def add_external_node(
         self,
         node_type,
         name,
@@ -142,12 +203,12 @@ class NodeManager:
         port,
         host,
         protocol,
-        external_node,
         default_alias=None,
     ):
         """Adding a node. Params:
         :param node_type: only valid for autodetect. Either BTC or ELM
-
+        This should only be used for an external node. Use add_internal_node for internal node
+        and if you have defined your own node type, use save_node directly to save the node (and create it yourself)
         """
         if not default_alias:
             node_alias = alias(name)
@@ -170,17 +231,24 @@ class NodeManager:
             port,
             host,
             protocol,
-            external_node,
             fullpath,
             node_type,
             self,
         )
-        logger.info(f"persisting {node} in add_node")
-        write_node(node, fullpath)
-        self.update()  # reload files
-        logger.info("Added new node {}".format(node.alias))
+        logger.debug(f"Persisting {node_alias} from an add_external_node call.")
+        self.nodes[node_alias] = node
+        self.save_node(node)
         return node
 
+    def save_node(self, node):
+        fullpath = (
+            node.fullpath
+            if hasattr(node, "fullpath")
+            else calc_fullpath(self.data_folder, node.alias)
+        )
+        write_node(node, fullpath)
+        logger.info("Added new node {}".format(node.alias))
+
     def add_internal_node(
         self,
         name,
@@ -189,6 +257,10 @@ class NodeManager:
         default_alias=None,
         datadir=None,
     ):
+        """Adding an internal node. Params:
+        This should only be used for internal nodes. Use add__External_node for external nodes
+        and if you have defined your own node-type, use save_node directly. to save the node (and create it yourself)
+        """
         if not default_alias:
             node_alias = alias(name)
         else:
@@ -218,19 +290,22 @@ class NodeManager:
             network,
             self.internal_bitcoind_version,
         )
-        logger.info(f"persisting {node} in add_internal_node")
-        write_node(node, fullpath)
-        self.update()  # reload files
-        logger.info("Added new internal node {}".format(node.alias))
+        self.nodes[node_alias] = node
         return node
 
     def delete_node(self, node, specter):
         logger.info("Deleting {}".format(node.alias))
-        # Delete files
-        delete_file(node.fullpath)
-        delete_file(node.fullpath + ".bkp")
-        if self._active_node == node.alias:
-            specter.update_active_node(next(iter(self.nodes.values())).alias)
-        del self.nodes[node.name]
-        self.update()
-        logger.info("Node {} was deleted successfully".format(node.alias))
+        try:
+            # Delete from wallet manager
+            del self.nodes[node.alias]
+            # Delete files
+            delete_file(node.fullpath)
+            delete_file(node.fullpath + ".bkp")
+            # Update the active node
+            if self._active_node == node.alias:
+                specter.update_active_node(
+                    next(iter(self.nodes.values())).alias
+                )  # This switches to the first node in the node list, which is usually the default node
+            logger.info("Node {} was deleted successfully".format(node.alias))
+        except KeyError:
+            raise SpecterError(f"{node.name} not found, node could not be deleted.")
