@@ -1,24 +1,184 @@
 import json
 import logging
 import os
+from os import path
+import shutil
+from typing import Type
 
 from embit.liquid.networks import get_network
+from flask import render_template
 from flask_babel import lazy_gettext as _
 from requests.exceptions import ConnectionError
-from .helpers import is_testnet, is_liquid
+
+from cryptoadvance.specter.devices.bitcoin_core import BitcoinCore, BitcoinCoreWatchOnly
+from cryptoadvance.specter.devices.elements_core import ElementsCore
+
+from .helpers import deep_update, is_liquid, is_testnet
 from .liquid.rpc import LiquidRPC
-from .persistence import write_node
+from .persistence import PersistentObject, write_node
 from .rpc import (
     BitcoinRPC,
     RpcError,
     autodetect_rpc_confs,
     get_default_datadir,
 )
+from .specter_error import SpecterError, BrokenCoreConnectionException
+from .device import Device
 
 logger = logging.getLogger(__name__)
 
 
-class Node:
+class AbstractNode(PersistentObject):
+    """This is a Node class worth deriving from. It tries to define as many attributes as possible which are needed but probably in a very
+    inefficient way, e.g. without any caching. Feel free to improve that in subclasses and you might get inspired by existing sublasses
+    """
+
+    # Many properties are convenience properties of informations, you get from the various info-rpc-callse, namely:
+    # * getblockchaininfo
+    # * getnetworkinfo
+    # * getmempoolinfo
+    # * uptime
+    # * getblockhash
+    # * scantxoutset
+
+    # So first, here are the one which return directly the content of one of those calls as dict:
+    @property
+    def info(self):
+        """Should be a combination of various info calls from Bitcoin Core. Could have:
+        * all the info from https://developer.bitcoin.org/reference/rpc/getblockchaininfo.html
+        * plus 'mempool_info' from https://developer.bitcoin.org/reference/rpc/getmempoolinfo.html
+        * plus uptime
+        * plus other stuff
+        We only implement a bare minimum here. See the Node-Impl for more
+
+        This method is exception-safe and returns an empty dict if the connection is broken
+        """
+        try:
+            res = [
+                r["result"]
+                for r in self.rpc.multi(
+                    [
+                        ("getblockchaininfo", None),
+                        ("getnetworkinfo", None),
+                        ("getmempoolinfo", None),
+                        ("uptime", None),
+                        ("getblockhash", 0),
+                        ("scantxoutset", "status", []),
+                    ]
+                )
+            ]
+            info = res[0]
+            info["mempool_info"] = res[2]
+            info["uptime"] = res[3]
+            return info
+        except BrokenCoreConnectionException:
+            return {}
+
+    @property
+    def network_info(self):
+        """https://developer.bitcoin.org/reference/rpc/getnetworkinfo.html
+        returns an almost empty dict if Broken Connection
+        """
+        try:
+            return self.rpc.getnetworkinfo()
+        except BrokenCoreConnectionException:
+            return {"subversion": "", "version": 999999}
+
+    @property
+    def uptime(self):
+        """https://developer.bitcoin.org/reference/rpc/uptime.html"""
+        return self.rpc.uptime()
+
+    # Now some even more convenient properties which provide often used handpicked information from those dicts
+
+    @property
+    def chain(self):
+        """current network name (main, test, regtest)
+        might have more values in elements/liquid
+        """
+        try:
+            return self.info.get("chain")
+        except BrokenCoreConnectionException:
+            # This is the most important part to signal that the node-connection is broken without throwin an Exception
+            return None
+
+    @property
+    def bitcoin_core_version_raw(self):
+        try:
+            return self.rpc.getnetworkinfo()["version"]
+        except BrokenCoreConnectionException:
+            # This is the most important part to signal that the node-connection is broken without throwin an Exception
+            return 99999
+
+    # ... and more derived properties which already calculate stuff based on those information
+
+    @property
+    def network_parameters(self):
+        return get_network(self.chain)
+
+    @property
+    def is_testnet(self):
+        return is_testnet(self.chain)
+
+    @property
+    def is_running(self):
+        if self.network_info["version"] == 999999:
+            logger.debug(f"Node is not running")
+            return False
+        else:
+            return True
+
+    def check_blockheight(self):
+        """check_blockheight is a method which is probably deprecated.
+        It should return True if there are new blocks available since check_info has been called
+        (which updates the cached _info[] dict)
+        """
+        raise NotImplemented(
+            "A Node Implementation need to implement the check_blockheight method"
+        )
+
+    def is_device_supported(self, device_class_or_device_instance):
+        """Lets the node deactivate specific devices. The parameter could be a device or a device_type
+            You have to check yourself if overriding this method.
+        e.g.
+        if device_instance_or_device_class.__class__ == type:
+            device_class = device_instance_or_device_class
+        else:
+            device_class = device_instance_or_device_class.__class__
+        # example:
+        # if BitcoinCore == device_class:
+        #    return False
+        return True
+        """
+        return True
+
+    def node_info_template(self):
+        """This should return the path to a Info template as string"""
+        return "node/components/bitcoin_core_info.jinja"
+
+    def node_logo_template(self):
+        """This should return the path to a Logo template as string
+        The template should contain the logo independent from the
+        status of the node. It's used in the node-selector
+        """
+        return "includes/sidebar/components/node_logo.jinja"
+
+    def node_connection_template(self):
+        """This should return the path to a connection template as string"""
+        return "includes/sidebar/components/node_connection.jinja"
+
+    def delete_wallet_file(self, wallet) -> bool:
+        """Deleting the wallet file located on the node. This only works if the node is on the same machine as Specter.
+        Returns True if the wallet file could be deleted, otherwise returns False.
+
+        In the case of an Abtract Node, we consider that method as an edge-case anyway and we just return False here.
+        That is the normal usage if you don't have access to the internals of your Bitcoin Core.
+        Overwrite as necessary.
+        """
+        return False
+
+
+class Node(AbstractNode):
     """A Node represents the connection to a Bitcoin and/or Liquid (Full-) node.
     It can be created via Constructor or from_json, and mainly it can give you A
     RPC-object to use the API.
@@ -27,6 +187,8 @@ class Node:
     One or many Nodes are managed via the NodeManager
     """
 
+    external_node = True
+
     def __init__(
         self,
         name,
@@ -38,7 +200,6 @@ class Node:
         port,
         host,
         protocol,
-        external_node,
         fullpath,
         node_type,
         manager,
@@ -54,7 +215,6 @@ class Node:
         :param port: usually something like 8332 for mainnet, 18332 for testnet, 18443 for Regtest, 38332 for signet
         :param host: domainname or ip-address. Don't add the protocol here
         :param protocol: Usually https or http
-        :param external_node: should be True for Node and False for InternalNode
         :param fullpath: it's assumed that you want to store it on disk AND decide about the fullpath upfront
         :param node_type: either "ELM" or "BTC", will impact autodetection (datadir and Env-vars)
         :param manager: A NodeManager instance which will get notified if the Node's name changes, the proxy_url will get copied from the manager as well
@@ -68,13 +228,15 @@ class Node:
         self.port = port
         self.host = host
         self.protocol = protocol
-        self.external_node = external_node
         self.fullpath = fullpath
         self._node_type = node_type
         self.manager = manager
         self.proxy_url = manager.proxy_url
         self.only_tor = manager.only_tor
-        self.rpc = self.get_rpc()
+        try:
+            self.rpc = self._get_rpc()
+        except BrokenCoreConnectionException:
+            self.rpc = None
         self._asset_labels = None
         self.check_info()
 
@@ -91,7 +253,6 @@ class Node:
         port = node_dict.get("port", None)
         host = node_dict.get("host", "localhost")
         protocol = node_dict.get("protocol", "http")
-        external_node = node_dict.get("external_node", True)
         fullpath = node_dict.get("fullpath", default_fullpath)
 
         return cls(
@@ -104,7 +265,6 @@ class Node:
             port,
             host,
             protocol,
-            external_node,
             fullpath,
             node_type,
             manager,
@@ -113,41 +273,47 @@ class Node:
     @property
     def json(self):
         """Get a json-representation of this Node"""
-        return {
-            "name": self.name,
-            "alias": self.alias,
-            "autodetect": self.autodetect,
-            "datadir": self.datadir,
-            "user": self.user,
-            "password": self.password,
-            "port": self.port,
-            "host": self.host,
-            "protocol": self.protocol,
-            "external_node": self.external_node,
-            "fullpath": self.fullpath,
-            "node_type": self.node_type,
-        }
+        node_json = super().json
+        return deep_update(
+            node_json,
+            {
+                "name": self.name,
+                "alias": self.alias,
+                "autodetect": self.autodetect,
+                "datadir": self.datadir,
+                "user": self.user,
+                "password": self.password,
+                "port": self.port,
+                "host": self.host,
+                "protocol": self.protocol,
+                "fullpath": self.fullpath,
+                "node_type": self.node_type,
+            },
+        )
 
-    def get_rpc(self):
-        """
-        Checks if config have changed, compares with old rpc
-        and returns new one if necessary
-        """
-        if hasattr(self, "rpc"):
-            rpc = self.rpc
+    def _get_rpc(self):
+        """Checks if configurations have changed, compares with old rpc
+        and returns new one if necessary.
+        Aims to be exception safe, returns None if rpc is not working"""
+        if hasattr(self, "_rpc"):
+            rpc = self._rpc
         else:
             rpc = None
         if self.autodetect:
-            if self.port:
-                rpc_conf_arr = autodetect_rpc_confs(
-                    self.node_type,
-                    datadir=os.path.expanduser(self.datadir),
-                    port=self.port,
-                )
-            else:
-                rpc_conf_arr = autodetect_rpc_confs(
-                    self.node_type, datadir=os.path.expanduser(self.datadir)
-                )
+            try:
+                if self.port:
+                    # autodetect_rpc_confs is trying a RPC call
+                    rpc_conf_arr = autodetect_rpc_confs(
+                        self.node_type,
+                        datadir=os.path.expanduser(self.datadir),
+                        port=self.port,
+                    )
+                else:
+                    rpc_conf_arr = autodetect_rpc_confs(
+                        self.node_type, datadir=os.path.expanduser(self.datadir)
+                    )
+            except BrokenCoreConnectionException:
+                return None
             if len(rpc_conf_arr) > 0:
                 rpc = BitcoinRPC(
                     **rpc_conf_arr[0], proxy_url=self.proxy_url, only_tor=self.only_tor
@@ -171,7 +337,7 @@ class Node:
             )
 
         if rpc == None:
-            logger.error(f"connection results to None in get_rpc")
+            logger.error(f"RPC connection is None in get_rpc. Returning None ...")
             return None
         # check if it's liquid
         try:
@@ -184,8 +350,8 @@ class Node:
                 return rpc  # The user is failing to configure correctly
             logger.error(rpce)
             return None
-        except ConnectionError as e:
-            logger.error(f"{e} while get_rpc")
+        except BrokenCoreConnectionException as bcce:
+            logger.error(f"{bcce} while get_rpc for {rpc}")
             return None
         except Exception as e:
             logger.exception(e)
@@ -194,7 +360,7 @@ class Node:
             return rpc
         else:
             logger.debug(
-                f"connection {rpc} fails test_connection() returning None in get_rpc"
+                f"Connection {rpc} fails test_connection() in get_rpc. Returning None ..."
             )
             return None
 
@@ -239,10 +405,14 @@ class Node:
             self.protocol = protocol
             update_rpc = True
         if update_rpc:
-            self.rpc = self.get_rpc()
-            if self.rpc and self.rpc.test_connection():
-                logger.debug(f"persisting {self} in update_rpc")
-                write_node(self, self.fullpath)
+            try:
+                self.rpc = self._get_rpc()
+                if self.rpc and self.rpc.test_connection():
+                    logger.debug(f"persisting {self} in update_rpc")
+                    write_node(self, self.fullpath)
+            except BrokenCoreConnectionException:
+                self._mark_node_as_broken()
+                return False
         self.check_info()
         return False if not self.rpc else self.rpc.test_connection()
 
@@ -251,11 +421,9 @@ class Node:
         self.name = new_name
         logger.info(f"persisting {self} in rename")
         write_node(self, self.fullpath)
-        self.manager.update()
 
     def check_info(self):
         self._is_configured = self.rpc is not None
-        self._is_running = False
         if self.rpc is not None and self.rpc.test_connection():
             try:
                 res = [
@@ -289,12 +457,8 @@ class Node:
                     self.utxorescanwallet = None
                 self._network_parameters = get_network(self.chain)
                 self._is_running = True
-            except Exception as e:
-                self._info = {"chain": None}
-                self._network_info = {"subversion": "", "version": 999999}
-                self._network_parameters = get_network("main")
-                logger.error(f"connection {self.rpc} could not suceed check_info")
-                logger.exception("Exception %s while check_info()" % e)
+            except BrokenCoreConnectionException:
+                self._mark_node_as_broken()
         else:
             if self.rpc is None:
                 logger.warning(f"connection of {self} is None in check_info")
@@ -315,24 +479,20 @@ class Node:
                     )
                 except Exception as e:
                     logger.exception(e)
-            self._info = {"chain": None}
-            self._network_info = {"subversion": "", "version": 999999}
-
-        if not self._is_running:
-            self._info["chain"] = None
+            self._mark_node_as_broken()
 
     def test_rpc(self):
         """tests the rpc-connection and returns a dict which helps
         to derive what might be wrong with the config
         ToDo: list an example here.
         """
-        rpc = self.get_rpc()
+        rpc = self._get_rpc()
         if rpc is None:
             return {
                 "out": "",
                 "err": _("Connection to node failed"),
                 "code": -1,
-                "tests": {},
+                "tests": {"connectable": False},
             }
         r = {}
         r["tests"] = {"connectable": False}
@@ -356,15 +516,14 @@ class Node:
                 r["err"] = "Wallets disabled"
 
             r["out"] = json.dumps(rpc.getblockchaininfo(), indent=4)
-        except ConnectionError as e:
-            logger.info("Caught an ConnectionError while test_rpc: %s", e)
-
+        except BrokenCoreConnectionException as bcce:
+            logger.info(f"Caught {bcce} while test_rpc")
             r["tests"]["connectable"] = False
             r["err"] = _("Failed to connect!")
             r["code"] = -1
         except RpcError as rpce:
             logger.info(
-                f"Caught an RpcError while test_rpc status_code: {rpce.status_code} error_code:{rpce.error_code}"
+                f"Caught an RpcError while test_rpc status_code: {rpce.status_code} error_code: {rpce.error_code}"
             )
             r["tests"]["connectable"] = True
             r["code"] = rpc.r.status_code
@@ -374,7 +533,7 @@ class Node:
             else:
                 r["err"] = str(rpce.status_code)
         except Exception as e:
-            logger.error(
+            logger.exception(
                 "Caught an exception of type {} while test_rpc: {}".format(
                     type(e), str(e)
                 )
@@ -388,6 +547,16 @@ class Node:
                 r["code"] = -1
         return r
 
+    def _mark_node_as_broken(self):
+        self._info = {"chain": None}
+        self._network_info = {"subversion": "", "version": 999999}
+        self._network_parameters = get_network("main")
+        logger.debug(
+            f"Node is not running, no RPC connection, check_info didn't succeed, setting RPC attribute to None ..."
+        )
+        self._info["chain"] = None
+        self.rpc = None
+
     def abortrescanutxo(self):
         """use this to abort a rescan as it stores some state while rescanning"""
         self.rpc.scantxoutset("abort", [])
@@ -407,9 +576,51 @@ class Node:
     def is_liquid(self):
         return is_liquid(self.chain)
 
+    def delete_wallet_file(self, wallet) -> bool:
+        """Deleting the wallet file located on the node. This only works if the node is on the same machine as Specter.
+        Returns True if the wallet file could be deleted, otherwise returns False."""
+        datadir = ""
+        if self.datadir == "":
+            # In case someone did not toggle the auto-detect but still used the default location.
+            # When you set up a new node and deactivate the auto-detect, the datadir is set to an empty string.
+            logger.debug(
+                f"The node datadir before get_default_datadir is: {self.datadir}"
+            )
+            datadir = get_default_datadir(self.node_type)
+            logger.debug(f"The node datadir after get_default_datadir is: {datadir}")
+        else:
+            datadir = self.datadir
+        wallet_file_removed = False
+        path = ""
+        # Check whether wallet was really unloaded
+        wallet_rpc_path = os.path.join(wallet.manager.rpc_path, wallet.alias)
+        # If we can unload the wallet via RPC it had not been unloaded properly before by the wallet manager
+        try:
+            self.rpc.unloadwallet(wallet_rpc_path)
+            raise SpecterError(
+                "Trying to delete the wallet file on the node but the wallet had not been unloaded properly."
+            )
+        except RpcError:
+            pass
+        if self.chain != "main":
+            path = os.path.join(datadir, f"{self.chain}/wallets", wallet_rpc_path)
+        else:
+            path = os.path.join(datadir, wallet_rpc_path)
+        try:
+            shutil.rmtree(path, ignore_errors=False)
+            logger.debug(f"Removing wallet file at: {path}")
+            wallet_file_removed = True
+        except FileNotFoundError:
+            logger.debug(f"Could not find any wallet file at: {path}")
+        return wallet_file_removed
+
     @property
     def is_running(self):
-        return self._is_running
+        if self._network_info["version"] == 999999:
+            logger.debug(f"Node is not running")
+            return False
+        else:
+            return True
 
     @property
     def is_configured(self):
@@ -451,14 +662,6 @@ class Node:
         except Exception as e:
             return False
 
-    @property
-    def chain(self):
-        return self.info["chain"]
-
-    @property
-    def is_testnet(self):
-        return is_testnet(self.chain)
-
     @property
     def asset_labels(self):
         if not self.is_liquid:
@@ -478,10 +681,12 @@ class Node:
 
     @property
     def rpc(self):
+        """Returns None if rpc is broken"""
         if not hasattr(self, "_rpc"):
-            return None
-        else:
-            return self._rpc
+            self._rpc = self._get_rpc()
+        elif self._rpc is None:
+            self._rpc = self._get_rpc()
+        return self._rpc
 
     @property
     def node_type(self):
@@ -490,6 +695,10 @@ class Node:
             return self._node_type
         return "BTC"
 
+    @property
+    def default_datadir(self):
+        return get_default_datadir(self.node_type)
+
     @rpc.setter
     def rpc(self, value):
         if hasattr(self, "_rpc") and self._rpc != value:
@@ -498,5 +707,7 @@ class Node:
             logger.debug(f"Updating {self}.rpc {self._rpc} with None (setter)")
         self._rpc = value
 
+    # UI specific stuff
+
     def __repr__(self) -> str:
         return f"<{self.__class__.__name__} name={self.name} fullpath={self.fullpath}>"
