@@ -9,7 +9,9 @@ import requests
 import urllib3
 
 from .helpers import is_ip_private
-from .specter_error import SpecterError, handle_exception
+from .specter_error import SpecterError, handle_exception, BrokenCoreConnectionException
+from urllib3.exceptions import NewConnectionError
+from requests.exceptions import ConnectionError
 
 logger = logging.getLogger(__name__)
 
@@ -189,10 +191,15 @@ def autodetect_rpc_confs(
             except requests.exceptions.RequestException as e:
                 pass
                 # no point in reporting that here
+            except SpecterError as e:
+                # Timeout
+                pass
             except RpcError:
                 pass
                 # have to make a list of acceptable exception unfortunately
                 # please enlarge if you find new ones
+    else:
+        logger.info(f"No candidates for BTC-connection autodetection found")
     return available_conf_arr
 
 
@@ -206,10 +213,26 @@ class RpcError(Exception):
         assert rpce.error_code == -32601
         assert rpce.error_msg == "Method not found"
     See for error_codes https://github.com/bitcoin/bitcoin/blob/v0.15.0.1/src/rpc/protocol.h#L32L87
+    You can create these RpcErrors via a response-object:
+        RpcError("some Message",response)
+    or directly via:
+        raise RpcError("some Message", status_code=500, error_code=-32601, error_msg="Requested wallet does not exist or is not loaded")
+    The message is swallowed if there is a proper error_msg.
     """
 
-    def __init__(self, message, response):
+    def __init__(
+        self, message, response=None, status_code=None, error_code=None, error_msg=None
+    ):
         super(Exception, self).__init__(message)
+        if response is not None:
+            self.init_via_response(response)
+        else:
+
+            self.status_code = status_code or 500
+            self.error_code = error_code or -99
+            self.error_msg = error_msg or str(self)
+
+    def init_via_response(self, response):
         self.status_code = 500  # default
         try:
             self.status_code = response.status_code
@@ -220,9 +243,9 @@ class RpcError(Exception):
         try:
             self.error_code = error["error"]["code"]
             self.error_msg = error["error"]["message"]
-        except Exception as e:
+        except Exception:
             self.error_code = -99
-            self.error = "UNKNOWN API-ERROR:%s" % response.text
+            self.error_msg = str(self) + " - UNKNOWN API-ERROR:%s" % response.text
 
 
 class BitcoinRPC:
@@ -366,6 +389,9 @@ class BitcoinRPC:
             r = self.session.post(
                 url, data=json.dumps(payload), headers=headers, timeout=timeout
             )
+        except (ConnectionError, NewConnectionError, ConnectionRefusedError) as ce:
+            raise BrokenCoreConnectionException()
+
         except (requests.exceptions.Timeout, urllib3.exceptions.ReadTimeoutError) as to:
             # Timeout is effectively one of the two:
             # ConnectTimeout: The request timed out while trying to connect to the remote server
@@ -386,6 +412,7 @@ class BitcoinRPC:
                     to,
                 )
             )
+            logger.exception(to)
             raise SpecterError(
                 "Timeout after {} secs while {} call({: <28}). Check the logs for more details.".format(
                     timeout,
