@@ -1,17 +1,23 @@
 import base64
 import binascii
+from datetime import date, datetime
 import cryptography
 import hashlib
 import json
 import logging
 import os
 import shutil
+import time
+import jwt
+import datetime
+import uuid
 
 from cryptography.fernet import Fernet
 from cryptography.hazmat.primitives import hashes
 from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
 
 from flask_login import UserMixin
+from flask import current_app as app
 
 from .specter_error import SpecterError, handle_exception
 from .persistence import read_json_file, write_json_file, delete_folder
@@ -70,6 +76,7 @@ class User(UserMixin):
         password_hash,
         config,
         specter,
+        jwt_tokens={},
         encrypted_user_secret=None,
         is_admin=False,
         services=[],
@@ -77,14 +84,15 @@ class User(UserMixin):
         self.id = id
         self.username = username
         self.password_hash = password_hash
+        self.jwt_tokens = jwt_tokens
         self.config = config
         self.encrypted_user_secret = encrypted_user_secret
         self.plaintext_user_secret = None
         self.is_admin = is_admin
         self.uid = specter.config["uid"]
         self.specter = specter
-        self.wallet_manager = None
-        self.device_manager = None
+        self._wallet_manager = None
+        self._device_manager = None
         self.manager = None
         self._services = services
 
@@ -98,6 +106,7 @@ class User(UserMixin):
             user_args = {
                 "id": user_dict["id"],
                 "username": user_dict["username"],
+                "jwt_tokens": user_dict.get("jwt_tokens", {}),
                 "password_hash": user_dict[
                     "password"
                 ],  # TODO: Migrate attr name to "password_hash"?
@@ -242,6 +251,7 @@ class User(UserMixin):
             "username": self.username,
             "password": self.password_hash,  # TODO: Migrate attr name to "password_hash"?
             "is_admin": self.is_admin,
+            "jwt_tokens": self.jwt_tokens,
             "encrypted_user_secret": self.encrypted_user_secret,
             "services": self.services,
         }
@@ -253,10 +263,24 @@ class User(UserMixin):
         self.check_device_manager()
         self.check_wallet_manager()
 
+    @property
+    def wallet_manager(self):
+        if self._wallet_manager is None:
+            self.check_wallet_manager()
+        assert self._wallet_manager is not None
+        return self._wallet_manager
+
+    @property
+    def device_manager(self):
+        if self._device_manager is None:
+            self.check_device_manager()
+        assert self._device_manager is not None
+        return self._device_manager
+
     def check_wallet_manager(self):
         """Updates wallet manager for this user"""
         # if chain, user or data folder changed
-        wallet_manager = self.wallet_manager
+        wallet_manager = self._wallet_manager
         wallets_rpcpath = "specter%s" % self.uid
         wallets_folder = os.path.join(
             self.specter.data_folder, f"wallets{self.folder_id}"
@@ -275,8 +299,11 @@ class User(UserMixin):
                 self.specter.chain,
                 self.device_manager,
                 path=wallets_rpcpath,
+                allow_threading_for_testing=self.specter.config["testing"][
+                    "allow_threading_for_testing"
+                ],
             )
-            self.wallet_manager = wallet_manager
+            self._wallet_manager = wallet_manager
         else:
             wallet_manager.update(
                 wallets_folder, self.specter.rpc, chain=self.specter.chain
@@ -287,10 +314,10 @@ class User(UserMixin):
         devices_folder = os.path.join(
             self.specter.data_folder, f"devices{self.folder_id}"
         )
-        if self.device_manager is None:
-            self.device_manager = DeviceManager(devices_folder)
+        if self._device_manager is None:
+            self._device_manager = DeviceManager(devices_folder)
         else:
-            self.device_manager.update(data_folder=devices_folder)
+            self._device_manager.update(data_folder=devices_folder)
 
     # TODO: Refactor this into UserManager
     def save_info(self, delete=False):
@@ -403,6 +430,112 @@ class User(UserMixin):
         # we delete wallet manager and device manager in save_info
         self.save_info(delete=True)
 
+    def add_jwt_token(
+        self, jwt_token_id, jwt_token, jwt_token_description, jwt_token_life
+    ):
+        # Adding a newly created JWT to the hashmap
+        self.jwt_tokens[jwt_token_id] = {}
+        self.jwt_tokens[jwt_token_id]["jwt_token"] = jwt_token
+        self.jwt_tokens[jwt_token_id]["jwt_token_description"] = jwt_token_description
+        self.jwt_tokens[jwt_token_id]["jwt_token_life"] = jwt_token_life
+        self.save_info()
+
+    def delete_jwt_token(self, jwt_token_id):
+        # Deleting a JWT from the hashmap
+        if jwt_token_id in self.jwt_tokens:
+            del self.jwt_tokens[jwt_token_id]
+            self.save_info()
+
+    def get_all_jwt_tokens_info(self):
+        # Getting all the JWT token IDs, descriptions and expiry times from the hashmap
+        return {
+            jwt_token_id: {
+                "jwt_token_description": self.jwt_tokens[jwt_token_id][
+                    "jwt_token_description"
+                ],
+                "jwt_token_life": self.jwt_tokens[jwt_token_id]["jwt_token_life"],
+                "jwt_token_remaining_life": self.jwt_token_life_remaining(jwt_token_id),
+            }
+            for jwt_token_id in self.jwt_tokens
+        }
+
+    def verify_jwt_token_id_and_jwt_token(self, jwt_token_id, jwt_token):
+        # Verifying the JWT token ID and JWT token
+        if jwt_token_id in self.jwt_tokens:
+            if self.jwt_tokens[jwt_token_id]["jwt_token"] == jwt_token:
+                return True
+        return False
+
+    def get_jwt_token(self, jwt_token_id):
+        # Getting a JWT token from the hashmap by ID
+        if jwt_token_id in self.jwt_tokens:
+            return {
+                "jwt_token_description": self.jwt_tokens[jwt_token_id][
+                    "jwt_token_description"
+                ],
+                "jwt_token_life": self.jwt_tokens[jwt_token_id]["jwt_token_life"],
+            }
+        return None
+
+    def get_jwt_token_by_token_id(self, jwt_token_id):
+        # Getting a JWT token from the hashmap by ID
+        if jwt_token_id in self.jwt_tokens:
+            return self.jwt_tokens[jwt_token_id]["jwt_token"]
+        return None
+
+    def get_jwt_token_life_by_token_id(self, jwt_token_id):
+        # Getting a JWT token life from the hashmap by ID
+        if jwt_token_id in self.jwt_tokens:
+            return self.jwt_tokens[jwt_token_id]["jwt_token_life"]
+        return None
+
+    def validate_jwt_token_description(self, jwt_token_description):
+        # Checking if the JWT token description is unique and not null
+        return (
+            jwt_token_description
+            not in [
+                self.jwt_tokens[jwt_token_id]["jwt_token_description"]
+                for jwt_token_id in self.jwt_tokens
+            ]
+            and jwt_token_description != ""
+        )
+
+    def jwt_token_life_remaining(self, jwt_token_id):
+        # Calculates the remaining life of a JWT token
+        jwt_token = self.get_jwt_token_by_token_id(jwt_token_id)
+        payload = jwt.decode(
+            jwt_token,
+            app.config["SECRET_KEY"],
+            algorithms=["HS256"],
+            options={"verify_signature": False},
+        )
+
+        if (payload["exp"] - time.time()) > 0:
+            return payload["exp"] - time.time()
+        return 0
+
+    @staticmethod
+    def generate_jwt_token(
+        username, jwt_token_id, jwt_token_description, jwt_token_life
+    ):
+        # Generates a JWT token for the user
+
+        # payload which will be encoded in the JWT token
+        payload = {
+            "username": username,
+            "jwt_token_id": jwt_token_id,
+            "jwt_token_description": jwt_token_description,
+            "exp": datetime.datetime.utcnow()
+            + datetime.timedelta(seconds=jwt_token_life),
+            "iat": datetime.datetime.utcnow(),
+        }
+        return jwt.encode(payload, app.config["SECRET_KEY"], algorithm="HS256")
+
+    @staticmethod
+    def generate_token_id():
+        # Generates a unique token id
+        return str(uuid.uuid4())
+
     def __eq__(self, other):
         if other == None:
             return False
